/*
 * $Id: XmlEditorKit.java,v 1.5 2003/10/01 17:38:09 edankert Exp $
 *
 * The contents of this file are subject to the Mozilla Public License 
 * Version 1.1 (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at 
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the License.
 *
 * The Original Code is eXchaNGeR browser code. (org.xngr.browser.*)
 *
 * The Initial Developer of the Original Code is Cladonia Ltd. Portions created 
 * by the Initial Developer are Copyright (C) 2002 the Initial Developer. 
 * All Rights Reserved. 
 *
 * Contributor(s): Edwin Dankert <edankert@cladonia.com>
 */
package org.xngr.browser.editor;

import java.awt.Font;
import java.awt.FontMetrics;

import javax.swing.text.*;

/**
 * The XML editor kit supports handling of editing XML content.  
 * It supports syntax highlighting, tab replacements and automatic 
 * indents.
 *
 * <p>
 * <b>Note:</b> The XML Editor package is based on the JavaEditorKit example as 
 * described in the article <i>'Customizing a Text Editor'</i> by <b>Timothy Prinzing</b>.
 * See: http://java.sun.com/products/jfc/tsc/articles/text/editor_kit/
 * </p>
 *
 * <p>
 * Change by ronnie: added getContext method.
 * </p>
 *
 * @version	$Revision: 1.5 $, $Date: 2003/10/01 17:38:09 $
 * @author Edwin Dankert <edankert@cladonia.com>
 */
public class XmlEditorKit extends DefaultEditorKit {

    private static Font font = null;

    private XmlContext context = null;
	private ViewFactory factory = null;
	private XmlEditorPane editor = null;
	
	/**
	 * Constructs the view factory and the Context.
	 */
    public XmlEditorKit( XmlEditorPane editor) {
		super();
		
		this.editor = editor;

		factory = new XmlViewFactory();
	    context = new XmlContext();
    }

    // added by ronnie
    public XmlContext getContext() {
	    return context;
    }

    /**
     * Get the MIME type of the data that this
     * kit represents support for. This kit supports
     * the type <code>text/xml</code>.
	 *
	 * @return the type.
     */
    public String getContentType() {
		return "text/xml";
    }

    public void setFont( Font font) {
    	this.font = font;
    	
    	context.setFont( font);
    }

    /**
     * Get the Font type for this kit.
     *
     * @return the font.
     */
    public Font getFont() {
		return font;
    }

    /**
     * Creates an uninitialized xml document.
     *
     * @return the document
     */
    public Document createDefaultDocument() {
		return new XmlDocument( editor);
    }

    /**
     * Fetches the XML factory that can produce views for 
	 * XML Documents.
     *
     * @return the XML factory
     */
    public final ViewFactory getViewFactory() {
		return factory;
    }
	
	/**
	 * A simple view factory implementation. 
	 */
	class XmlViewFactory implements ViewFactory {
		// Creates the XML View.
		public View create(Element elem) {
			return new XmlView( context, elem);
		}
	}
}







