/*
 * $Id: XmlEditorPane.java,v 1.5 2003/10/01 17:38:09 edankert Exp $
 *
 * The contents of this file are subject to the Mozilla Public License 
 * Version 1.1 (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at 
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the License.
 *
 * The Original Code is eXchaNGeR browser code. (org.xngr.browser.*)
 *
 * The Initial Developer of the Original Code is Cladonia Ltd. Portions created 
 * by the Initial Developer are Copyright (C) 2002 the Initial Developer. 
 * All Rights Reserved. 
 *
 * Contributor(s): Edwin Dankert <edankert@cladonia.com>
 */
package org.xngr.browser.editor;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;

import javax.swing.JEditorPane;
import javax.swing.text.Document;
import javax.swing.event.DocumentListener;
import javax.swing.event.DocumentEvent;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

/**
 * An extension of the JEditorPane that allows for syntax-highlighted 
 * text in the XML format.
 *
 * <p>
 * <b>Note:</b> The XML Editor package is based on the JavaEditorKit example as 
 * described in the article <i>'Customizing a Text Editor'</i> by <b>Tim Prinzing</b>.
 * See: http://java.sun.com/products/jfc/tsc/articles/text/editor_kit/
 * </p>
 *
 * @version	$Revision: 1.5 $, $Date: 2003/10/01 17:38:09 $
 * @author Edwin Dankert <edankert@cladonia.com>
 */
public class XmlEditorPane extends JEditorPane {
	private boolean tagCompletion = true;
	private static int defaultTabSize = 4;
	private int spaces = 4;
	private XmlEditorKit kit = null;

	/**
	 * Constructs the XML editor pane, sets the font, 
	 * background color, content-type and the editor kit.
	 */
	public XmlEditorPane() {
		kit = new XmlEditorKit( this);

		setEditorKitForContentType( kit.getContentType(), kit);
		setBackground( Color.white);

		setContentType( kit.getContentType());
		super.setFont( kit.getFont());

		setEditable( true);
	}
	
	public boolean isTagCompletion() {
		return tagCompletion;
	}

	public void setFont( Font font) {
		if ( kit != null) {
			kit.setFont( font);
		}
		super.setFont( font);
	}

	public void setTabSize( int size) {
	    defaultTabSize = size;
	    Document doc = getDocument();

	    if (doc != null) {
	        int old = getTabSize();

	        doc.putProperty( XmlDocument.tabSizeAttribute, new Integer(size));
	        firePropertyChange( "tabSize", old, size);
	    }
	}

	/**
	 * Gets the number of characters used to expand tabs.  If the document is
	 * null or doesn't have a tab setting, return a default of 8.
	 *
	 * @return the number of characters
	 */
	public int getTabSize() {
	    int size = defaultTabSize;
	    Document doc = getDocument();

	    if (doc != null) {
	        Integer i = (Integer) doc.getProperty( XmlDocument.tabSizeAttribute);
	        if (i != null) {
	            size = i.intValue();
	        }
	    }

	    return size;
	}

	public void setText( String text) {
		XmlDocument doc = (XmlDocument)getDocument();
		doc.setLoading( true);
		super.setText( text);
		doc.setLoading( false);
	}

	public int getSpaces() {
		return spaces;
	}
}
