package sk.robotics.rlcs.agents.beh;

import sk.robotics.rlcs.agentspace.Agent;
import sk.robotics.rlcs.data.Position;

import java.awt.geom.Point2D;

/**
 *  Agent which moves a robot towards another robot.
 *  It just copies position of one robot to destination of another one.
 *  ApproachAgent (with optional AvoidAgent) should be used to realize
 *  the actual movement.
 */
public class FollowAgent extends Agent {

	public static class Cfg extends Agent.Cfg {
		/** Name of block with current destination of the following robot.
		 *  @block.type java.awt.geom.Point2D
		 *  @block.access write */
		public @Required @NoDefault String blockDestination = "destination/robot?";

		/** Name of block with actual position of the followed robot.
		 *  @block.type sk.robotics.rlcs.data.Position
		 *  @block.access read */
		public @Required @NoDefault String blockFollowedPos = "actualPos/robot?";

		public Cfg() {
			// default activation
			timerPeriodMs = 100;
		}
	}
	private final Cfg cfg;

	public FollowAgent(Cfg cfg) {
		super(cfg);
		this.cfg = cfg;
	}

	@Override protected void senseSelectAct() {
		// read position of followed robot
		Position pos = (Position)space.read(cfg.blockFollowedPos);

		// write destination of following robot
		space.write(cfg.blockDestination, pos == null ? null : pos.asPoint2D());
	}
}
