package sk.robotics.rlcs.agents.image;

import java.awt.image.*;

/**
 *  Image filter, that converts the image to grayscale.
 *  The input must be a 3-band RGB or 4-band RGBA image.
 */
public class GrayScaleAgent extends ImageFilterAgent {

	public static class Cfg extends ImageFilterAgent.Cfg {
		/** Multiplier for the red color component. */
		public double red = 0.2125;

		/** Multiplier for the green color component. */
		public double green = 0.7154;

		/** Multiplier for the blue color component. */
		public double blue = 0.0721;

		public Cfg() {
			// forced output type
			outputImageType = ImageType.GRAY;
		}

		@Override public void validate() {
			if (!"GRAY".equals(outputImageType)) {
				throw new IllegalArgumentException("outputImageType myst be GRAY!");
			}
			super.validate();
		}
	}
	private Cfg cfg;

	public GrayScaleAgent(Cfg cfg) {
		super(cfg);
		this.cfg = cfg;
	}

	@Override protected void filter(BufferedImage source, BufferedImage target) {
		Raster in = source.getRaster();
		WritableRaster out = target.getRaster();

		for (int x = 0; x < source.getWidth(); ++x) {
			for (int y = 0; y < source.getHeight(); ++y) {
				int r = in.getSample(x, y, 0);
				int g = in.getSample(x, y, 1);
				int b = in.getSample(x, y, 2);

				int gray = (int) ((cfg.red * r) + (cfg.green * g) + (cfg.blue * b));
				for (int n = 0; n < out.getNumBands(); ++n) {
					out.setSample(x, y, n, gray);
				}
			}
		}
	}
}
