package sk.robotics.rlcs.agents.image;

import java.awt.image.*;

/**
 *  Sobel image filter. Typically used for edge detection.
 */
public class SobelAgent extends ImageFilterAgent {

	public static class Cfg extends ImageFilterAgent.Cfg {
		/** Perform horizontal Sobel. */
		public boolean horizontal = true;

		/** Perform vertical Sobel. */
		public boolean vertical = true;

		@Override public void validate() {
			// output image must be of same type and size like input image
			if (outputImageType != null || outputWidth != null || outputHeight != null) {
				throw new IllegalArgumentException("Custom output image type and/or size not supported!");
			}
			super.validate();
		}
	}
	private Cfg cfg;

	public SobelAgent(Cfg cfg) {
		super(cfg);
		this.cfg = cfg;
	}

	@Override protected void filter(BufferedImage source, BufferedImage target) {
		Raster in = source.getRaster();
		WritableRaster out = target.getRaster();

		for (int bnd = 0; bnd < in.getNumBands(); ++bnd) {
			for (int x = 1, y = 1; y < source.getHeight() - 1; ++y, x = 1) {
				int s1 = in.getSample(x-1, y-1, bnd);
				int s2 = in.getSample(x,   y-1, bnd);
				int s4 = in.getSample(x-1, y,   bnd);
				int s5 = in.getSample(x,   y,   bnd);
				int s7 = in.getSample(x-1, y+1, bnd);
				int s8 = in.getSample(x,   y+1, bnd);

				for (; x < source.getWidth() - 1; ++x) {
					int s3 = in.getSample(x+1, y-1, bnd);
					int s6 = in.getSample(x+1, y,   bnd);
					int s9 = in.getSample(x+1, y+1, bnd);

					int sobel = 0;
					if (cfg.horizontal) {
						sobel += Math.abs((s1 + 2*s4 + s7) - (s3 + 2*s6 + s9));
					}
					if (cfg.vertical) {
						sobel += Math.abs((s1 + 2*s2 + s3) - (s7 + 2*s8 + s9));
					}
					sobel = Math.min(sobel, 255);

					out.setSample(x, y, bnd, sobel);

					s1 = s2;
					s2 = s3;
					s4 = s5;
					s5 = s6;
					s7 = s8;
					s8 = s9;
				}
			}
		}
	}
}
