package sk.robotics.rlcs.agents.image;

import java.awt.image.BufferedImage;
import java.awt.image.AffineTransformOp;
import java.awt.geom.AffineTransform;

import java.util.*;

/**
 *  Affine transformation image filter.
 */
public class TransformAgent extends ImageFilterAgent {

	public static class Cfg extends ImageFilterAgent.Cfg {
		/** Interpolation type. */
		public Interpolation interpolation = Interpolation.BILINEAR;
		public static enum Interpolation {
			NEAREST(AffineTransformOp.TYPE_NEAREST_NEIGHBOR),
			BILINEAR(AffineTransformOp.TYPE_BILINEAR),
			BICUBIC(AffineTransformOp.TYPE_BICUBIC);

			private final int value;
			private Interpolation(int x) { value = x; }

			public int getValue() { return value; }
		}

		/** X-scale item of the transformation matrix. See java.awt.geom.AffineTransform. */
		public double scaleX = 1;

		/** Y-scale item of the transformation matrix. See java.awt.geom.AffineTransform. */
		public double scaleY = 1;

		/** X-shear item of the transformation matrix. See java.awt.geom.AffineTransform. */
		public double shearX = 0;

		/** Y-shear item of the transformation matrix. See java.awt.geom.AffineTransform. */
		public double shearY = 0;

		/** X-translate item of the transformation matrix. See java.awt.geom.AffineTransform. */
		public double translateX = 0;

		/** Y-translate item of the transformation matrix. See java.awt.geom.AffineTransform. */
		public double translateY = 0;

		@Override public void validate() {
			// custom output image type must not be set
			if (outputImageType != null) {
				throw new IllegalArgumentException("Custom output image type not supported!");
			}
			super.validate();
		}

		// hide the outputImageType field in web editor
		public Set<String> getHiddenFields() {
			return hideFields(super.getHiddenFields(), "outputImageType");
		}
	}

	// the transformation operator
	protected AffineTransformOp transformOp;

	public TransformAgent(Cfg cfg) {
		super(cfg);

		// construct the transform operator
		AffineTransform transform = new AffineTransform(
			cfg.scaleX, cfg.shearY, cfg.shearX, cfg.scaleY, cfg.translateX, cfg.translateY);
		transformOp = new AffineTransformOp(transform, cfg.interpolation.getValue());
	}

	@Override protected void filter(BufferedImage source, BufferedImage target) {
		transformOp.filter(source, target);
	}
}
