package sk.robotics.rlcs.agents.image;

import java.awt.image.*;

/**
 *  Treshold filter.
 *  Accepts gray-scale images on input, and produces black and white output.
 */
public class TresholdAgent extends ImageFilterAgent {

	public static class Cfg extends ImageFilterAgent.Cfg {
		/** The treshold value. All pixels greater value will
		 *  get converted to white, others to black color. */
		public int treshold = 100;

		public Cfg() {
			// forced input/output types
			inputImageType = ImageType.GRAY;
			outputImageType = ImageType.BW;
		}

		@Override public void validate() {
			if (!"GRAY".equals(inputImageType) || !"BW".equals(outputImageType)) {
				throw new IllegalArgumentException("inputImageType and outputImageType must be GRAY and BW!");
			}
			super.validate();
		}
	}
	private Cfg cfg;

	public TresholdAgent(Cfg cfg) {
		super(cfg);
		this.cfg = cfg;
	}

	@Override protected void filter(BufferedImage source, BufferedImage target) {
		Raster in = source.getRaster();
		WritableRaster out = target.getRaster();

		for (int x = 0; x < source.getWidth(); ++x) {
			for (int y = 0; y < source.getHeight(); ++y) {
				int pixel = (in.getSample(x, y, 0) < cfg.treshold) ? 0 : 1;
				out.setSample(x, y, 0, pixel);
			}
		}
	}
}
