package sk.robotics.rlcs.agents.os;

import sk.robotics.rlcs.agentspace.Agent;
import java.io.File;

/**
 *  Simple agent for touching a file.
 *  It sets the file's last-modified timestamp.
 * <p>
 *  This agent uses java.io.File.setLastModified(), which on unix works only if
 *  the caller is owner of the file. It that is not the case, ExecuterAgent may
 *  be used on instead, executing the touch command.
 */
public class FileToucherAgent extends Agent {

	public static class Cfg extends Agent.Cfg {
		/** Path to the file to touch. */
		public @Required @NoDefault String filePath = "/var/www/imgcalib/do_calib.touch";
	}

	// the File object, corresponding to the file path from config
	private File file;

	public FileToucherAgent(Cfg cfg) {
		super(cfg);
		file = new File(cfg.filePath);
	}

	@Override protected void senseSelectAct() {
		// touch the file
		if (!file.setLastModified(System.currentTimeMillis())) {
			log(0, "failed to touch the file: " + file.getPath());
		}
	}
}
