package sk.robotics.rlcs.agents.robot;

import sk.robotics.rlcs.agentspace.Agent;
import sk.robotics.rlcs.agentspace.Space;
import sk.robotics.rlcs.data.Movement;
import sk.robotics.rlcs.data.Position;
import sk.robotics.rlcs.data.Collision;
import sk.robotics.rlcs.constants.TurtleConstants;

import java.awt.geom.Point2D;

/**
 *  Agent for moving a robot towards destination, stored in space.
 *  This agent turns the robot until it heads towards the destination point,
 *  and then moves it forward until it reaches the destination.
 */
public class ApproachAgent extends MoveAgentBase {

	public static class Cfg extends MoveAgentBase.Cfg {}

	public ApproachAgent(Cfg cfg) {
		super(cfg);
	}

	@Override protected Movement selectMove(Position pos, Point2D dest) {
		// get desired movement direction
		double dir = pos.directionTo(dest);

		// select next movement
		double dirDiff = -pos.headingDiff(dir);
		Movement.Direction move;
		if (Math.abs(dirDiff) > cfg.directionTolerance) {
			// current heading is bad, let's turn
			if (dirDiff > 0) move = Movement.Direction.RT;
			else             move = Movement.Direction.LT;
		} else {
			// current heading is ok, let's go forward
			move = Movement.Direction.FD;
		}

		if (move == Movement.Direction.FD) {
			log(5, "heading for destination [" + dest.getX() + " " + dest.getY() + "], dist: " + pos.distance(dest));
		} else {
			log(5, "turning to destination [" + dest.getX() + " " + dest.getY() + "], dirDiff: " + dirDiff);
		}

		// move, at full speed
		return Movement.valueOf(move, 99, 99);
	}
}
