package sk.robotics.rlcs.agents.space;

import sk.robotics.rlcs.agentspace.Agent;
import sk.robotics.rlcs.agentspace.Space;

/**
 *  Simple agent which copies content of one block into another.
 */
public class CopierAgent extends Agent {

	public static class Cfg extends Agent.Cfg {
		/** Name of the source block.
		 *  @block.access read */
		public @Required String blockSource;

		/** Name of the target block.
		 *  @block.access write */
		public @Required String blockTarget;

		/** Priority of writing into the target block.
		 *  If not set, priority of the source block is used. */
		public Float priority;

		/** Validity of writing into the target block.
		 *  Written data will be valid for this period, given in milliseconds.
		 *  Zero or negative value means infinity. If not set, validity is
		 *  copied from the source block. */
		public Long validity;

		/** Copy also null. This does make sense sometimes, to deny lower-priority writes. */
		public Boolean copyNull = false;

		public Cfg() {
			// default activation
			triggers = "$blockSource";
		}
	}
	private final Cfg cfg;

	public CopierAgent(Cfg cfg) {
		super(cfg);
		this.cfg = cfg;
	}

	@Override protected void senseSelectAct() {
		// read the source block
		Space.Block block = space.getBlock(cfg.blockSource);
		if (block == null) return;
		if (block.getValue() == null && !cfg.copyNull) return;

		// setup priority and validity
		float priority = (cfg.priority == null) ? block.getPriority() : cfg.priority;
		long validFrom = (cfg.validity == null) ? block.getValidFrom() : System.currentTimeMillis();
		long validTo = (cfg.validity == null) ? block.getValidTo() : validFrom + cfg.validity;

		// write the target block
		space.write(cfg.blockTarget, block.getValue(), validFrom, validTo, priority);
	}
}
