package sk.robotics.rlcs.security;

import sk.robotics.rlcs.agentspace.Agent;

/**
 *  This class provides a way to execude some code in a restricted thread.
 *  The thread will have either no permissions at all, or only those of
 *  a specified agent class. Of course, only privileged threads are allowed
 *  to create (and execute) threads of this class.
 */
public class Restricted extends Thread {

	/**
	 *  Creates a restricted thread object.
	 *  Call exec method to (synchronously) execute the thread.
	 */
	public Restricted(Runnable target, String name, Class agentClass) {
		super(target, name);
		this.agentClass = agentClass;
	}

	/**
	 *  Get agent class, associated with this thread.
	 */
	public Class getAgentClass() {
		return agentClass;
	}

	/**
	 *  Executes the thread, waiting for it to finish before returning.
	 *  If it throws any RuntimeException, this method will rethrow it.
	 */
	public void exec() {
		start();
		for (;;) {
			try { join(); break; }
			catch (InterruptedException e) {}
		}
		if (exception != null) {
			throw new RuntimeException(exception);
		}
	}

	// executes the target, and stores thrown RuntimeException, if any
	public void run() {
		try { super.run(); }
		catch (RuntimeException e) {
			exception = e;
		}
	}

	// exception thrown by the target
	private volatile RuntimeException exception;

	// agent class, associated with this thread
	private Class agentClass;

}
