package sk.robotics.rlcs.web.spaceview;

import java.net.*;
import java.io.*;
import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 *  Applet for loading and displaying values from VLCS's space.
 */
public class SpaceView extends JApplet {
	// default port where VLCS listens
	// (default host is the one serving the page, and both
	//  are overridable by parameters in HTML <applet> tag)
	private final static int DEFAULT_REMOTE_PORT = 35759;

	// real host and port
	private String vlcsHost;
	private int vlcsPort;

	// GUI components
	private JTextField textBlockName;
	private JButton btnLoad;
	private JTextArea textResult;

	/**
	 *  Initialization.
	 */
	public void init() {
		// determine the remote host and port
		vlcsHost = getParameter("host");
		if (vlcsHost == null || vlcsHost.length() == 0) {
			vlcsHost = getDocumentBase().getHost();
		}
		try { vlcsPort = Integer.parseInt(getParameter("port")); }
		catch (Exception e) { vlcsPort = DEFAULT_REMOTE_PORT; }

		// init the GUI
		JPanel top = new JPanel();
		top.setLayout(new BorderLayout());
		top.add(new JLabel("Block name: "), BorderLayout.WEST);
		top.add(textBlockName = new JTextField(), BorderLayout.CENTER);
		top.add(btnLoad = new JButton("Load"), BorderLayout.EAST);

		setLayout(new BorderLayout());
		add(top, BorderLayout.NORTH);
		add(new JScrollPane(textResult = new JTextArea()), BorderLayout.CENTER);

		// load button click
		ActionListener actionListener = new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				load();
			}
		};
		btnLoad.addActionListener(actionListener);
		textBlockName.addActionListener(actionListener);
	}

	// load from space
	private void load() {
		// get block name
		String blockName = textBlockName.getText();
		if (blockName.length() == 0) {
			JOptionPane.showMessageDialog(this, "Fill the block name!", "Error", JOptionPane.ERROR_MESSAGE);
			return;
		}

		Socket sock = null;
		try {
			// connect to VLCS
			sock = new Socket(vlcsHost, vlcsPort);

			// send request
			Writer writer = new OutputStreamWriter(sock.getOutputStream());
			writer.write("READSPACE " + blockName + "\n");
			writer.flush();

			// read reply
			ByteArrayOutputStream reply = new ByteArrayOutputStream(1024);
			byte[] buffer = new byte[1024];
			BufferedInputStream input = new BufferedInputStream(sock.getInputStream());
			for (;;) {
				int rd = input.read(buffer);
				if (rd == -1) break;
				reply.write(buffer, 0, rd);
			}

			// display the result
			textResult.setText(reply.toString());
		}
		catch (Exception e) {
			e.printStackTrace(System.err);
			JOptionPane.showMessageDialog(this, e, "Error", JOptionPane.ERROR_MESSAGE);
		}

		// close the socket
		try { if (sock != null) sock.close(); }
		catch (IOException e) {}
	}
}
