package sk.robotics.vlabcfg;

/** Access to the configuration of the virtuallab. */
public class VlabCfg {

  /** Get value of variable. 
   * @param varname name of the variable
   * @return value of the variable, or "#undefined", if variable doesn't exist, or "#error" in case of error (i.e. daemon cannot be reached) */
  public static synchronized native String get(String varname);

  /** Get value of variable. 
   * @param varname name of the variable
   * @param fresh set fresh=1 if you want to reload from referred files, if any (@N(filename) references), otherwise use fresh=0
   * @return value of the variable, or "#undefined", if variable doesn't exist, or "#error" in case of error (i.e. daemon cannot be reached) */
  public static synchronized native String get(String varname, int fresh);

  /** Get value of variable. 
   * @param varname name of the variable
   * @param defval if defval is not null, and if there is a problem, or variable undefined, the String specified in defval is returned
   * @param fresh set fresh=1 if you want to reload from referred files, if any (@N(filename) references), otherwise use fresh=0
   * @return value of the variable, or "#undefined", if variable doesn't exist, or "#error" in case of error (i.e. daemon cannot be reached) */
  public static synchronized native String get(String varname, String defval, int fresh);

  /** Get int value of variable.
   * @param varname name of the variable
   * @param fresh set to 1, if you want to reload file references, otherwise leave at 0
   * @return int value of the variable, or -1 if undefined or error occurs */
  public static synchronized native int getInt(String varname, int fresh);

  /** Get double value of variable.
   * @param varname name of the variable
   * @param fresh set to 1, if you want to reload file references, otherwise leave at 0
   * @return double value of the variable, or -1 if undefined or error occurs */
  public static synchronized native double getDouble(String varname, int fresh);

  /** Get definition of the variable.
   * @param varname name of the variable
   * @return definition of the value (i.e. the raw value without being evaluated) */
  public static synchronized native String def(String varname);

  /** Set value of variable.
   * @param varname name of the variable (it must already exist - to create new variables, use vlabcfg utility with -edit option)
   * @param value new value for the variable - may contain references to other variables in form $(variable), and references to files in form @N(filename), where N is a line of the text file to retrieve, and filename can again contain variable references
   * @param password the password is required for write-access to the configuration settings
   * @return "#ok", if successful, otherwise "#error" */
  public static synchronized native String set(String varname, String value, String password);

  /** Test if daemon is running and reachable.
   * @return true, if the daemon is running and reachable, false otherwise */
  public static synchronized native boolean ready();

  static
  {
    System.loadLibrary("vlabcfgjava");
  }

}
